
using System;
using System.Security;
using System.Security.Permissions;


// Enum describing the level of toasting of a piece of bread.
public enum ToastLevel
{
    Uncooked,
    LightBrown,
    MediumBrown,
    DarkBrown,
    Burnt
}


[Serializable]
sealed public class ToasterPermission : CodeAccessPermission,
                                        IUnrestrictedPermission
{
    // Permission state fields. Normally we'd make these private
    // and provide public property accessors, but for simplicity's
    // sake we'll just make them public (none are read-only anyway).

    // The ability to exclusively control the toaster.
    public bool ExclusiveAccess;

    // The ability to read information about toast state.
    public bool Information;

    // The ability to eject toast.
    public bool Eject;

    // The maximum level to which auto-eject can be set for the
    // toaster.
    public ToastLevel ToastLevel;

    // Provide the usual constructor taking a PermissionState for
    // consistency with other permission types.
    public ToasterPermission(PermissionState state)
    {
        if (state == PermissionState.Unrestricted)
        {
            ExclusiveAccess = true;
            Information = true;
            Eject = true;
            ToastLevel = ToastLevel.Burnt;
        }
    }

    // Create a deep copy of the current permission.
    public override IPermission Copy()
    {
        ToasterPermission perm;

        // Create a vanilla permission with no state.
        perm = new ToasterPermission(PermissionState.None);

        // Copy state fields from the current instance into the
        // new instance.
        perm.ExclusiveAccess = ExclusiveAccess;
        perm.Information = Information;
        perm.Eject = Eject;
        perm.ToastLevel = ToastLevel;

        // Return the new copy.
        return perm;
    }

    // Compute a new permission with all the state common between
    // the current and supplied permission.
    public override IPermission Intersect(IPermission target)
    {
        // An input of null is a special case (it represents a
        // permission with no state). There is obviously no common
        // state in such cases and we should return null to represent
        // this.
        if (target == null)
            return null;

        // Cast target into a ToasterPermission for ease of use.
        // Note that this will throw an InvalidCastException if
        // someone mistakenly passes us the wrong permission instance.
        ToasterPermission other = (ToasterPermission)target;

        ToasterPermission result;

        // Create a vanilla permission with no state.
        result = new ToasterPermission(PermissionState.None);

        // Consider shared state for each state field in turn.
        result.ExclusiveAccess = ExclusiveAccess && other.ExclusiveAccess;
        result.Information = Information && other.Information;
        result.Eject = Eject && other.Eject;

        // Shared state for ToastLevel is the lesser of the two.
        result.ToastLevel = ToastLevel < other.ToastLevel ?
            ToastLevel : other.ToastLevel;

        // Return the shared state instance.
        return result;
    }

    // Compute a new permission with all the state available from
    // either the current or supplied permission.
    public override IPermission Union(IPermission target)
    {
        // An input of null is a special case (it represents a
        // permission with no state). We can just return a copy of
        // the current permission in this case.
        if (target == null)
            return Copy();

        // Cast target into a ToasterPermission for ease of use.
        // Note that this will throw an InvalidCastException if
        // someone mistakenly passes us the wrong permission instance.
        ToasterPermission other = (ToasterPermission)target;

        ToasterPermission result;

        // Create a vanilla permission with no state.
        result = new ToasterPermission(PermissionState.None);

        // Consider shared state for each state field in turn.
        result.ExclusiveAccess = ExclusiveAccess || other.ExclusiveAccess;
        result.Information = Information || other.Information;
        result.Eject = Eject || other.Eject;

        // The union of state for ToastLevel is the most permissive
        // of the two.
        result.ToastLevel = ToastLevel < other.ToastLevel ?
            other.ToastLevel : ToastLevel;

        // Return the shared state instance.
        return result;
    }

    // Compute whether the current permission is a subset of the
    // supplied permission.
    public override bool IsSubsetOf(IPermission target)
    {
        // An input of null is a special case (it represents a
        // permission with no state). We can only be a subset if
        // we're in a similar empty state.
        if (target == null)
            return !ExclusiveAccess &&
                   !Information &&
                   !Eject &&
                   ToastLevel == ToastLevel.Uncooked;

        // Cast target into a ToasterPermission for ease of use.
        // Note that this will throw an InvalidCastException if
        // someone mistakenly passes us the wrong permission instance.
        ToasterPermission other = (ToasterPermission)target;

        // For each state field that's in a non-empty state, check
        // that the state is mirrored (or present at a greater
        // permissive state) in the target.

        if (ExclusiveAccess && !other.ExclusiveAccess)
            return false;

        if (Information && !other.Information)
            return false;

        if (Eject && !other.Eject)
            return false;

        if (ToastLevel > other.ToastLevel)
            return false;

        // Checked all state, we must be a subset.
        return true;
    }

    // Return true if the permission is in its most permissive state.
    public bool IsUnrestricted()
    {
        return ExclusiveAccess &&
               Information &&
               Eject &&
               ToastLevel == ToastLevel.Burnt;
    }

    // Convert the current permission to a SecurityElement (or tree
    // of elements).
    public override SecurityElement ToXml()
    {
        // Create new element, tag name must always be "IPermission",
        // element text is not needed.
        SecurityElement elem = new SecurityElement("IPermission");

        // Determine assembly qualified full type name of our
        // permission class (the security uses this to locate and
        // load the class).
        String name = typeof(ToasterPermission).AssemblyQualifiedName;

        // Add attributes for the class name and protocol version
        // (currently always "1") to the element.
        elem.AddAttribute("class", name);
        elem.AddAttribute("version", "1");

        if (IsUnrestricted())
        {
            // Create an abbreviated encoding for unrestricted
            // instances of this permission.
            elem.AddAttribute("Unrestricted", Boolean.TrueString);
        }
        else
        {
            // Encode each state field as an attribute of the element.
            // Only bother adding elements for state in the non-default
            // state, this makes the encoded format more compact.

            if (ExclusiveAccess)
                elem.AddAttribute("ExclusiveAccess", Boolean.TrueString);

            if (Information)
                elem.AddAttribute("Information", Boolean.TrueString);

            if (Eject)
                elem.AddAttribute("Eject", Boolean.TrueString);

            if (ToastLevel != ToastLevel.Uncooked)
                elem.AddAttribute("ToastLevel", ToastLevel.ToString());
        }

        // Return the completed element.
        return elem;
    }

    // Convert a SecurityElement (or tree of elements) to a permission
    // instance.
    public override void FromXml(SecurityElement e)
    {
        // Check whether whether we have an unrestricted instance.
        String unrestricted = e.Attribute("Unrestricted");
        if (unrestricted != null &&
            Boolean.Parse(unrestricted))
        {
            ExclusiveAccess = true;
            Information = true;
            Eject = true;
            ToastLevel = ToastLevel.Burnt;
            return;
        }

        // Ensure we start from a clean base state.
        ExclusiveAccess = false;
        Information = false;
        Eject = false;
        ToastLevel = ToastLevel.Uncooked;

        // Check the value of each attribute which encodes a state
        // field.
        String value;

        value = e.Attribute("ExclusiveAccess");
        if (value != null)
            ExclusiveAccess = Boolean.Parse(value);

        value = e.Attribute("Information");
        if (value != null)
            Information = Boolean.Parse(value);

        value = e.Attribute("Eject");
        if (value != null)
            Eject = Boolean.Parse(value);

        value = e.Attribute("ToastLevel");
        if (value != null)
            ToastLevel = (ToastLevel)Enum.Parse(typeof(ToastLevel),
                                                value, true);
    }
}
